`old.att` <-
function (obj, formula, data, model="lm",family="binomial") 
{
 if((class(obj)[1] != "cem.match") && (class(obj)[1] != "cem.match.list"))
  stop("Argument `obj' must be a `cem.match' or `cem.match.list' object") 
 if(!(model %in% c("lm","glm")))
  stop("please, only use `lm' or `glm'")
   
 if(class(obj)[1] == "cem.match"){
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w))
  else
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w,family=family))
  aa <- summary(out)
  out <- list(model = t(aa$coefficients))
  class(out) <- "cem.att"
  return(out)
 }
 
 if(class(data) != "list")
  stop("Argument `data' must be a list of `data.frame's")

 n.cems <- length(obj) - 1

 if(length(data) != n.cems)
  stop("lengths of `cem.match.list' object and `data' do not match")
  
 est <- vector(n.cems, mode="list")
 for(i in 1:n.cems){ 
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w))
  else
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w,family=family))
  aa <- summary(out)
  est[[i]] <- aa$coefficients 
 }
 qoi <- numeric(dim(est[[1]])[1])
 seq <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  qoi <- qoi + est[[i]][,1]
  seq <- seq + est[[i]][,2]^2
 }
 
 qoi <- qoi/n.cems
 seq <- seq/n.cems

 s2 <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  s2 <- s2 + (est[[i]][,1] - qoi)^2
 }
 s2 <- s2/(n.cems-1)

 S <- sqrt(seq + s2*(1+1/n.cems))

 mat <- rbind(qoi, S)
 rownames(mat) <- c("Estimate", "Std. Error")
 
 out <- list(mult=est, model = mat) 
 class(out) <- "cem.att"
 out
}



print.cem.att <- function(x,...){
 if(!is.null(x$tab)){
  cat("\n")
  print(x$tab)
  }
 if(!is.null(x$fixed.cem)){
  cat("\nFixed effects model on the CEM matched subsample:\n\n")
  print(x$fixed.cem)
  }
 if(!is.null(x$random.cem)){
   cat("\nRandom effects model on CEM matched subsample:\n\n")
   print(x$random.cem)
  }

 if(!is.null(x$random.all)){
   cat("\nRandom effects model on the whole sample:\n\n")
   print(x$random.all)
  }
  
}

plot.cem.att <- function(x,...){
 if(!is.null(x$cem.model)){
  plot(x$cem.model, main="Fixed effects model on the CEM matched subsample")
  }
  
 if(!is.null(x$ranef.cem)){
    p1 <- plot(x$ranef.cem,main="Random effects model on CEM matched subsample")
    print(p1,new=TRUE)
  }

 if(!is.null(x$ranef.all)){
   p2 <- plot(x$ranef.all, main="Random effects model on the whole sample")
   print(p2,new=TRUE)
  }
  
}


############### THE LME STUFF ##############


`att` <-
function (obj, formula, data, model="lm",family="binomial", random=FALSE) 
{

 if((class(obj)[1] != "cem.match") && (class(obj)[1] != "cem.match.list"))
  stop("Argument `obj' must be a `cem.match' or `cem.match.list' object") 
 if(!(model %in% c("lm","glm")))
  stop("please, only use `lm' or `glm'")
  random.cem <- NULL
  random.all <- NULL
  ranef.cem <- NULL
  ranef.all <- NULL
  fixed.cem <- NULL
  if(class(obj)[1] == "cem.match"){

  if(model=="lm"){
   out <- do.call(model,list(formula=formula, data=data, weights=obj$w))

   if(random){     
    require(nlme, quietly=TRUE)
    data1 <- data
    data1$allID <- factor(obj$strata)
    data1$cemID <- factor(obj$mstrata)
	data1$matched <- obj$matched
    f1 <- formula(sprintf(" ~ %s | allID",obj$treatment))   
    f2 <- formula(sprintf(" ~ %s | cemID",obj$treatment))   
# all data
   rand.all <- lme(update(formula, ~ . -cemID -allID - matched), data=data1, random = f1)
#   rand.all <- lme(formula, data=data1, random = f1)
   random.all <- t(summary(rand.all)$tTable)
   rownames(random.all) <- c("Estimate", "Std. Error", "DF", "t value", "p-value")
   ranef.all <- random.effects(rand.all)
# restricted
   rand.cem <- lme(update(formula, ~ . -cemID -allID - matched), data=data1, random = f2, subset=matched==TRUE)
#   rand.cem <- lme(formula, data=data1, random = f2, subset=matched==TRUE)
   random.cem <- t(summary(rand.cem)$tTable)
   rownames(random.cem) <- c("Estimate", "Std. Error", "DF", "t value", "p-value")
   ranef.cem <- random.effects(rand.cem)
   }
  } else {
   out <- do.call(model, list(formula=formula, data=data, weights=obj$w,family=family))
  }
  fixed.cem <- t(summary(out)$coefficients)
  rownames(fixed.cem) <-  c("Estimate", "Std. Error",  "t value", "p-value")
  out <- list(fixed.cem = fixed.cem, random.cem=random.cem, random.all=random.all, tab=obj$tab, 
   ranef.cem=ranef.cem, ranef.all=ranef.all, cem.model = out)
  class(out) <- "cem.att"
  return(out)
 }
 
 if(class(data) != "list")
  stop("Argument `data' must be a list of `data.frame's")

 n.cems <- length(obj) - 1

 if(length(data) != n.cems)
  stop("lengths of `cem.match.list' object and `data' do not match")
  
 est <- vector(n.cems, mode="list")
 for(i in 1:n.cems){ 
  if(model=="lm")
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w))
  else
   out <- do.call(model, list(formula=formula, data=data[[i]], weights=obj[[i]]$w,family=family))
  aa <- summary(out)
  est[[i]] <- aa$coefficients 
 }
 qoi <- numeric(dim(est[[1]])[1])
 seq <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  qoi <- qoi + est[[i]][,1]
  seq <- seq + est[[i]][,2]^2
 }
 
 qoi <- qoi/n.cems
 seq <- seq/n.cems

 s2 <- numeric(dim(est[[1]])[1])
 for(i in 1:n.cems){
  s2 <- s2 + (est[[i]][,1] - qoi)^2
 }
 s2 <- s2/(n.cems-1)

 S <- sqrt(seq + s2*(1+1/n.cems))

 mat <- rbind(qoi, S)
 rownames(mat) <- c("Estimate", "Std. Error")
 
 out <- list(mult=est, model = mat) 
 class(out) <- "cem.att"
 out
}

